/**
 * Author: Henrik Hedman
 * Created: den 04 december 2003 17:00:23
 * Function: The main window.
 **/

import java.awt.*; // to access components needed to build a gui 
import java.awt.event.*; // needed for event handling (buttons, menus)
import java.io.*; // needed for reading and writing to file
import javax.swing.*; // needed in order to use Swing components
import java.net.*; // used to manage URLs
import java.applet.*; // needed to use audio among other things

/**
 * The main class in the Schedule Program:
 *  initiates everyting and creates the JFrame.
 **/
public class SchemaBokning extends JFrame
{
    /**
     * Create Boolean to check if logged in and if admin is logged in
     * Create JPanel for layout
     * Create JButton for action
     * Create JLabel for information
     * Create Menu for easy access
     * Create Container to hold JPanels
     * Create JTextfield to log in
     * Constants: height & width of schedulebuttons
     *            height & width of labels
     *            The default color on labels
     **/
    
    private boolean adminLogged = false;
    private boolean inLoggad = false;
    private boolean userPanel = false;
    private boolean schemaPanel = false;
    private boolean schemaKnapp = false;
    private Container contentpane;
    private JPanel leftLayoutPanel;
    private JPanel rightLayoutPanel;
    private JPanel northLayoutPanel;
    private JPanel southLayoutPanel;
    private JPanel centerLayoutPanel;
    private JLabel schemaNameTextLabel;
    private JLabel schemaMellanrum;
    private JLabel schemaMellanrum2;
    private JTextField schemaNameTextfield;
    private JButton schemaLoggaInButton;
    private JButton schemaAvslutaButton;
    private JButton schemaLoggaUtButton;
    private JButton schemaUsersButton;
    private JButton schemaSchemaButton;
    
    private JMenuBar schemaMenuBar;
    private JMenu fileMenu;
    private JMenu aboutMenu;
    private JMenuItem quitMenuItem;
    private JMenuItem undoMenuItem;
    private JMenuItem redoMenuItem;
    private JMenuItem aboutMenuItem;
    private JMenuItem helpMenuItem;

    private JLabel schemaPicture;
    private ImageIcon nkpg = new ImageIcon("icons/nkpg.png");
    
    private AudioClip errorSound;
    
    /** set button height **/
    protected static final int BUTTONHEIGHT = 20;
    /** set button width **/    
    protected static final int BUTTONWIDTH = 100;
    /** set default color **/   
    protected static final Color DEFAULTCOLOR = new Color(200, 210, 245);
    
    /** Create object of class UserHandler to import its panel **/
    public UserHandler userHand = new UserHandler( this );
    /** Create object of class SchemaCreator to import its panel **/
    public SchemaCreator schemaCreator = new SchemaCreator(this);
 
    private SchemaBokning() {
        //initiate sound
        try {errorSound = Applet.newAudioClip(new File("errorsound.wav").toURL());}
            catch(MalformedURLException e){}
        //settings for the JFrame
        setSize(800,600);
        setLocation(getToolkit().getScreenSize().width/2 - getSize().width/2, getToolkit().getScreenSize().height/2 - getSize().height/2);
        setDefaultCloseOperation(EXIT_ON_CLOSE); // adds a exit on close command
        setTitle("Schema 1.0");
            
        //------- initiate all the components, set some colours and behaviour ------
        //picture
        schemaPicture = new JLabel(nkpg);
        schemaPicture.setAlignmentX(CENTER_ALIGNMENT);
        schemaPicture.setToolTipText("Skolan");
        //login text
        schemaNameTextLabel = new JLabel("Logga in", JLabel.CENTER);
        schemaNameTextLabel.setAlignmentX(CENTER_ALIGNMENT);
        schemaNameTextLabel.setBackground(DEFAULTCOLOR);
        schemaNameTextLabel.setOpaque(true);
        schemaNameTextLabel.setMaximumSize(new Dimension(BUTTONWIDTH, 15));
        //create a space
        schemaMellanrum = new JLabel(" ");
        schemaMellanrum.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaMellanrum.setAlignmentX(CENTER_ALIGNMENT);     
        schemaMellanrum2 = new JLabel(" ");     
        schemaMellanrum2.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaMellanrum2.setAlignmentX(CENTER_ALIGNMENT);
        //log in box
        schemaNameTextfield = new JTextField("");
        schemaNameTextfield.setForeground(Color.black);
        schemaNameTextfield.setBackground(Color.white);
        schemaNameTextfield.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaNameTextfield.setAlignmentX(CENTER_ALIGNMENT);
        //log in button
        schemaLoggaInButton = new JButton("Logga in");
        schemaLoggaInButton.setMnemonic(KeyEvent.VK_N);
        schemaLoggaInButton.setToolTipText("Logga in");
        schemaLoggaInButton.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaLoggaInButton.setAlignmentX(CENTER_ALIGNMENT);
        //log out button
        schemaLoggaUtButton = new JButton("Logga ut");
        schemaLoggaUtButton.setMnemonic(KeyEvent.VK_U);
        schemaLoggaUtButton.setToolTipText("Logga ut");
        schemaLoggaUtButton.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaLoggaUtButton.setAlignmentX(CENTER_ALIGNMENT);
        //exit button
        schemaAvslutaButton = new JButton("Avsluta", new ImageIcon(Toolkit.getDefaultToolkit().getImage("icons/Exit_smal.png").getScaledInstance(10, 10, Image.SCALE_SMOOTH)));
        schemaAvslutaButton.setMnemonic(KeyEvent.VK_X);
        schemaAvslutaButton.setToolTipText("Avsluta");
        schemaAvslutaButton.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaAvslutaButton.setAlignmentX(CENTER_ALIGNMENT);
        //user button
        schemaUsersButton = new JButton("Anvndare");
        schemaUsersButton.setToolTipText("Anvndare");
        schemaUsersButton.setMnemonic(KeyEvent.VK_A);
        schemaUsersButton.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaUsersButton.setAlignmentX(CENTER_ALIGNMENT);
        //schedule button
        schemaSchemaButton = new JButton("Schema");
        schemaSchemaButton.setToolTipText("Schema");
        schemaSchemaButton.setMnemonic(KeyEvent.VK_S);
        schemaSchemaButton.setMaximumSize(new Dimension(BUTTONWIDTH, BUTTONHEIGHT));
        schemaSchemaButton.setAlignmentX(CENTER_ALIGNMENT);
        
        // initiate the menu
        schemaMenuBar = new JMenuBar();
        fileMenu = new JMenu("File");
        quitMenuItem = new JMenuItem("Avsluta");
        undoMenuItem = new JMenuItem("ngra");
        redoMenuItem = new JMenuItem("Upprepa");
        aboutMenu = new JMenu("Hjlp");
        helpMenuItem = new JMenuItem("Hjlp");
        aboutMenuItem = new JMenuItem("Om");
				
        // add the menubar, its menues and alternatives
        setJMenuBar(schemaMenuBar);
        schemaMenuBar.add(fileMenu);
        fileMenu.add(quitMenuItem);
        schemaMenuBar.add(aboutMenu);
        aboutMenu.add(helpMenuItem);
        aboutMenu.add(aboutMenuItem);
        
        // add listeners to the components that can generate actions
        quitMenuItem.addActionListener(schemaListener);
        helpMenuItem.addActionListener(schemaListener);
        aboutMenuItem.addActionListener(schemaListener);    
        schemaLoggaInButton.addActionListener(schemaListener);
        schemaAvslutaButton.addActionListener(schemaListener);
        schemaLoggaUtButton.addActionListener(schemaListener);
        schemaUsersButton.addActionListener(schemaListener);
        schemaSchemaButton.addActionListener(schemaListener);
        schemaNameTextfield.addActionListener(schemaListener);

        // ----------------------------- LAYOUT ---------------------------------
        // choose LayoutManager
        contentpane = getContentPane();
        contentpane.setLayout(new BorderLayout());
        // initiate the panels that builds the layout, choose LayoutManager
        leftLayoutPanel = new JPanel();
        rightLayoutPanel = new JPanel();
        northLayoutPanel = new JPanel();
        southLayoutPanel = new JPanel();
        centerLayoutPanel = new JPanel();
        //set the layout in the panels
        northLayoutPanel.setLayout(new BoxLayout(northLayoutPanel, BoxLayout.Y_AXIS));
        southLayoutPanel.setLayout(new BoxLayout(southLayoutPanel, BoxLayout.Y_AXIS));
        centerLayoutPanel.setLayout(new BoxLayout(centerLayoutPanel, BoxLayout.Y_AXIS));
        rightLayoutPanel.setLayout(null);
        leftLayoutPanel.setLayout(new BorderLayout());
        leftLayoutPanel.add(northLayoutPanel, BorderLayout.NORTH);
        leftLayoutPanel.add(southLayoutPanel, BorderLayout.SOUTH);
        leftLayoutPanel.add(centerLayoutPanel, BorderLayout.CENTER);
        //add to contentpane      
        contentpane.add(leftLayoutPanel, BorderLayout.WEST);
        contentpane.add(rightLayoutPanel, BorderLayout.CENTER);
        //set static objects (i.e. exit button, picture)
        northLayoutPanel.add(Box.createRigidArea(new Dimension(150, 10)));
        northLayoutPanel.add(schemaPicture);
        northLayoutPanel.add(Box.createRigidArea(new Dimension(150, 10)));
        southLayoutPanel.add(Box.createRigidArea(new Dimension(150, 10)));
        southLayoutPanel.add(schemaAvslutaButton);
        rightLayoutPanel.setBackground(DEFAULTCOLOR);
        northLayoutPanel.setBackground(DEFAULTCOLOR);
        southLayoutPanel.setBackground(DEFAULTCOLOR);
        centerLayoutPanel.setBackground(DEFAULTCOLOR);
        // open the JFrame with this method
		schemaCreator.updateButtons();
        openWindow();

		
    }
    /**
    *Method to open the JFrame
    *Initiate Buttons, Textfield, Panels, Colors
    **/
    public void openWindow()
    {
        //All the buttons and panels in the layout that can change
        northLayoutPanel.remove(schemaUsersButton);
        northLayoutPanel.remove(schemaMellanrum);
        if(inLoggad)
        {
            schemaCreator.setEnable(true);
            //add & remove buttons
            northLayoutPanel.remove(schemaLoggaInButton);
            northLayoutPanel.remove(schemaNameTextfield);
            northLayoutPanel.add(schemaLoggaUtButton);
            northLayoutPanel.add(schemaMellanrum2);         
            //set text
            schemaNameTextLabel.setText(userHand.currentUser.name + " [lrare]");
            schemaNameTextfield.setText("");
            //color
            schemaNameTextLabel.setBackground(userHand.currentUser.userColor.getLight());
        }
        else
        {
            schemaCreator.setEnable(false);
            //add & remove buttons
            northLayoutPanel.remove(schemaMellanrum2);          
            northLayoutPanel.remove(schemaLoggaUtButton);
            northLayoutPanel.remove(schemaUsersButton);
            northLayoutPanel.remove(schemaSchemaButton);
            northLayoutPanel.add(schemaNameTextLabel);
            northLayoutPanel.add(schemaNameTextfield);
            northLayoutPanel.add(schemaLoggaInButton);
            //color
            schemaNameTextLabel.setText("Logga in");
            schemaNameTextfield.setText("");
            schemaNameTextLabel.setBackground(DEFAULTCOLOR);
        }
        //admin buttons
        if(adminLogged) 
        {
            schemaNameTextLabel.setText(userHand.currentUser.name + " [admin]");
            northLayoutPanel.add(schemaSchemaButton);
            northLayoutPanel.add(schemaUsersButton);
        }
        //right panel layout
        if(schemaPanel && schemaKnapp == false && userPanel == false)
        {
            //show schedule
            rightLayoutPanel.remove(userHand.panel);
            rightLayoutPanel.add(schemaCreator.schemalayoutPanel);
            centerLayoutPanel.remove(schemaCreator.schemaAppPanel);
        }
        else if(schemaPanel && schemaKnapp && userPanel == false)
        {
            //show appointment
            centerLayoutPanel.add(schemaCreator.schemaAppPanel);
        }
        else if(userPanel && schemaPanel == false)
        {
            //user panel
            rightLayoutPanel.remove(schemaCreator.schemalayoutPanel);
            rightLayoutPanel.add(userHand.panel);
            centerLayoutPanel.remove(schemaCreator.schemaAppPanel);
        }
        else
        {
            //when not logged in, show schedule
            rightLayoutPanel.remove(schemaCreator.schemalayoutPanel);
            rightLayoutPanel.remove(userHand.panel);
            rightLayoutPanel.add(schemaCreator.schemalayoutPanel);
            centerLayoutPanel.remove(schemaCreator.schemaAppPanel);
        }
        //set bounds for panels
        userHand.panel.setBounds(0, 0, 585, 540); 
        schemaCreator.schemalayoutPanel.setBounds(0, 0, 585, 540); 
        //Open window
        setResizable(false);
        setVisible(true);
        
    }
    
    /**
     * Method to show appointment called from schemaCreator.
     **/
    public void schemaApp(boolean open)
    {
            if(open)
            {
                schemaKnapp = true;
                schemaPanel = true;
                openWindow();
                schemaPanel = false;
                schemaKnapp = false;
            }
            else
            {
                schemaPanel = false;
                schemaKnapp = false;
                openWindow();
            }
    }

    /**
     * ActionListener for buttons in the left panel and for the menu
     **/
    ActionListener schemaListener = new ActionListener() {
        public void actionPerformed(ActionEvent e) 
        {
            //log in
            if (e.getSource() == schemaLoggaInButton || e.getSource() == schemaNameTextfield) 
            {
                //if admin
                adminLogged = false;
                if(userHand.login(schemaNameTextfield.getText().trim()))
                {
                    inLoggad = true;
                    if(userHand.currentUser.admin) //admin eller ej
                        adminLogged = true;
                }
                else //if user
                    {
                        errorSound.play();
                        JOptionPane.showMessageDialog(null, " Anvndarnamnet r felaktigt!", "Fel anvndare", JOptionPane.ERROR_MESSAGE);
                    }
                openWindow();         
            }
            //log out
            if (e.getSource() == schemaLoggaUtButton) 
            {
                schemaCreator.updateButtons();
                adminLogged = false;
                inLoggad = false;
                userPanel = false;
                schemaPanel = false;
                userHand.logout();
                openWindow();
            }
            //show user panel
            if (e.getSource() == schemaUsersButton) 
            {
                schemaPanel = false;
                userPanel = true;
                openWindow();
            }
            //show schedule
            if (e.getSource() == schemaSchemaButton) 
            {
                schemaPanel = true;
                userPanel = false;
                schemaCreator.updateButtons();
                openWindow();
            }
            //listernet on the menues
            if (e.getSource( ) == aboutMenuItem)
            {
                // show about
                JOptionPane.showMessageDialog(null, " Databasprojekt TNM018\n" + 
                    " Programmerare:\n" +
                    "     Henrik Hedman - henhe894@student.liu.se\n" +
                    "     Jonas Hellsten - jonhe289@student.liu.se\n" + 
                    "     Peter Asplund - petas874@student.liu.se\n", "About", JOptionPane.PLAIN_MESSAGE, nkpg);
            }
            
            if (e.getSource( ) == helpMenuItem)
            {
                // show help
                JOptionPane.showMessageDialog(null, " Klicka p schemat fr att boka en tid", "Hjlp?", JOptionPane.INFORMATION_MESSAGE);           
            }
            //exit from menu or exit button
            if (e.getSource() == schemaAvslutaButton || e.getSource() == quitMenuItem) 
            {
                int avslut = JOptionPane.showConfirmDialog(null, " Vill du avsluta programmet?","Avsluta?",JOptionPane.ERROR_MESSAGE);
                if(avslut == 0)
                {
                    // quit the program
                    dispose();
                    System.exit(0);
                }
                else;
            }
        }
    };
    
    /**
     * Main Method to initiate the constructor and set the Look and Feel
     **/
    public static void main(String args[]) throws IOException {
        JFrame.setDefaultLookAndFeelDecorated(true);
        // set the "lookandfeel", you can choose different types
        try { UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel"); }
        catch (Exception e) { }
        
        // create the JFrame that our program is an extension of
        SchemaBokning nyttschema = new SchemaBokning();
    }      
}

