/**
 * Author: Peter Asplund
 * Created: den 04 december 2003 17:00:23
 * Function: Creating and drawing the schedule
 **/
	
import javax.swing.*;			//Swing Components
import java.awt.*;				//Graphical Components
import java.awt.event.*;	// needed for event handling (buttons, menus)
import java.sql.*;

class SchemaCreator extends JPanel
{
	/**
	*Create Button Array, x=days, y=timeslot
	*Create JPanel for layout
	*Create Schedule Components: buttons, day and time labels
	*Constants: height & width of schedulebuttons
	*						height & width of labels
	*						height & width of icons
	*						The default slot when time is available
	**/
	private final static int BUTTONHEIGHT = 105, BUTTONWIDTH = 95, DAYHEIGHT = 20, HOURWIDTH = 50, ICON_X = 10, ICON_Y = 10;
	private String tempToolTipText, tempSubject;
	private int day = 0, time = 0;
	private int activeWeek = 6;
	
	Connection con;
	Statement stmt;	
	
	private JButton buttons[][] = new JButton[5][5];
	public JPanel schemalayoutPanel;
	private JLabel dayTextArea[][] = new JLabel[6][6];
	private JLabel currentWeek;
	private JButton prevWeek = new JButton("",
		new ImageIcon(Toolkit.getDefaultToolkit().getImage("icons/prev.png").getScaledInstance(ICON_X, ICON_Y, Image.SCALE_SMOOTH)));
	private JButton nextWeek = new JButton("",
		new ImageIcon(Toolkit.getDefaultToolkit().getImage("icons/next.png").getScaledInstance(ICON_X, ICON_Y, Image.SCALE_SMOOTH)));
	private SchemaBokning schemaBokning;
	public UserHandler userHandler;

	/**
	*Create JPanel for Appointmentbooking
	*including buttons, textfields, dropdown menus
	**/
	
	public JPanel schemaAppPanel;
	private JLabel appDay;
	private JLabel appTime;
	private JLabel appSubject;
	private JLabel appToolTip;
	private JLabel appTextFormat;
	private JComboBox appSubjectDropDown;
	private JTextField appToolTipText;
	private JButton appOK;
	private JButton appAbort;	
	private JButton appCancel;
	private String[] userSubjects;
	private String[] weekday = {"Mndag","Tisdag","Onsdag","Torsdag","Fredag"};
	private String[] daytime = {"08-10","10-12","","13-15","15-17"};
	
	/**
	*Constructor
	*Enables JPanel, buttons, places labels etc.
	*Recieves: SchemaBokning (main class)
	*Returns: nothing
	**/
	 
	public SchemaCreator(SchemaBokning args){
		schemaBokning = args;
		userHandler = schemaBokning.userHand;
		schemalayoutPanel = new JPanel();
		schemalayoutPanel.setLayout(null);	//Layout for panel is set by x&y coord.
		schemaAppPanel = new JPanel();
		schemaAppPanel.setLayout(null);
		appOK = new JButton("OK", 
			new ImageIcon(Toolkit.getDefaultToolkit().getImage("icons/ok_smal.png").getScaledInstance(ICON_X, ICON_Y, Image.SCALE_SMOOTH)));
		appAbort = new JButton("Avbryt", 
			new ImageIcon(Toolkit.getDefaultToolkit().getImage("icons/cancel_smal.png").getScaledInstance(ICON_X, ICON_Y, Image.SCALE_SMOOTH)));
		appCancel = new JButton("Aboka");
		appDay = new JLabel("");
		appTime = new JLabel("");
		appSubject = new JLabel("mne: ");
		appToolTip = new JLabel("Info: ");
		appTextFormat = new JLabel("Skriv information:");
		appSubjectDropDown = new JComboBox();
		appToolTipText = new JTextField("");
		
		appDay.setBounds(9, 20, 100, 15);
		appTime.setBounds(9, 35, 100, 15);
		appSubject.setBounds(9, 50, 120, 15);
		appSubjectDropDown.setBounds(9, 65, 120, 20);
		appToolTip.setBounds(9, 100, 160, 15);
		appTextFormat.setBounds(9, 115, 100, 15);
		appToolTipText.setBounds(9, 135, 160, 20);
		appCancel.setBounds(60, 170, 80, 25);
		appOK.setBounds(10, 200, 85, 20);
		appAbort.setBounds(100, 200, 85, 20);

		prevWeek.setToolTipText("Fregende vecka");
		nextWeek.setToolTipText("Nsta vecka");
		
		/**
		*Set background according to defaultcolor
		**/

		schemalayoutPanel.setBackground(schemaBokning.DEFAULTCOLOR);
		schemaAppPanel.setBackground(schemaBokning.DEFAULTCOLOR);	
		
		/**
		*Initiate Labels days in dayTextArea (time and day)
		*Five slots of each
		*[0][y] = days
		*[x][0] = time of day
		**/

		dayTextArea[0][0] = new JLabel();
		dayTextArea[1][0] = new JLabel("Mndag");
		dayTextArea[2][0] = new JLabel("Tisdag");
		dayTextArea[3][0] = new JLabel("Onsdag");
		dayTextArea[4][0] = new JLabel("Torsdag");
		dayTextArea[5][0] = new JLabel("Fredag");
		dayTextArea[0][1] = new JLabel("08-10");
		dayTextArea[0][2] = new JLabel("10-12");
		dayTextArea[0][3] = new JLabel("12-13");
		dayTextArea[0][4] = new JLabel("13-15");
		dayTextArea[0][5] = new JLabel("15-17");
	
		/**
		*Place empty square in left corner of schedule
		*Height and Width set by constants HOURWIDTH and DAYHEIGHT
		**/
		
		dayTextArea[0][0].setBounds(25, 25, HOURWIDTH, DAYHEIGHT);
		dayTextArea[0][0].setBackground(schemaBokning.DEFAULTCOLOR);
		
		/**
		*Place nextWeek and PrevWeek Buttons at bottom of schedule
		**/
		
		prevWeek.setBounds(75, 500, 105, 20);
		currentWeek = new JLabel("Aktuell vecka: " + activeWeek);
		currentWeek.setBounds(270, 500, 105, 20);
		nextWeek.setBounds(445, 500, 105, 20);
		
		//Place Days and Times in Schedule
		for(int x = 1; x < 6; x++){
			dayTextArea[x][0].setBounds(25+HOURWIDTH+BUTTONWIDTH*(x-1), 25, BUTTONWIDTH, DAYHEIGHT);
			dayTextArea[x][0].setBackground(schemaBokning.DEFAULTCOLOR);		
			dayTextArea[x][0].setAlignmentX(RIGHT_ALIGNMENT);
		}			
		for(int y = 1; y < 6; y++){
			if(y < 3)
				dayTextArea[0][y].setBounds(25, 25+DAYHEIGHT+BUTTONHEIGHT*(y-1), HOURWIDTH, BUTTONHEIGHT);
			else {
				if(y == 3)
					dayTextArea[0][y].setBounds(25, 25+DAYHEIGHT+BUTTONHEIGHT*(y-1), HOURWIDTH, 30);
				else
					dayTextArea[0][y].setBounds(25, 30+25+DAYHEIGHT+BUTTONHEIGHT*(y-2), HOURWIDTH, BUTTONHEIGHT);														
			}
			dayTextArea[0][y].setBackground(schemaBokning.DEFAULTCOLOR);
			dayTextArea[0][y].setAlignmentX(CENTER_ALIGNMENT);
		}
		
		//Create Schedule
		createButtonGrid();

	}
		
	//Method to call when creating buttons and schedule
	public void createButtonGrid() {
		//Initiate Buttongrid
		for (int dag = 0; dag < 5; dag++) {
			for (int tid = 0; tid < 5; tid++){
						
				String subject = "ledig";
				String tooltiptext = "ledig";
				String label = "";			
						
				if (tid == 0 || tid == 1){
					buttons[dag][tid] = new JButton(subject);
					buttons[dag][tid].setToolTipText(tooltiptext);
					buttons[dag][tid].setBounds(25+BUTTONWIDTH*dag+HOURWIDTH, 25+BUTTONHEIGHT*tid+DAYHEIGHT, BUTTONWIDTH, BUTTONHEIGHT);	
					buttons[dag][tid].setBackground(Color.WHITE);
					}
				if (tid == 2){
					buttons[dag][tid] = new JButton("LUNCH");
					buttons[dag][tid].setToolTipText("LUNCH");
					buttons[dag][tid].setEnabled(false);
					buttons[dag][tid].setBounds(25+BUTTONWIDTH*dag+HOURWIDTH, 25+BUTTONHEIGHT*2+DAYHEIGHT, BUTTONWIDTH, 30);
					}
				if (tid == 3 || tid == 4){
					buttons[dag][tid] = new JButton(subject);
					buttons[dag][tid].setToolTipText(tooltiptext);				
					buttons[dag][tid].setBounds(25+BUTTONWIDTH*dag+HOURWIDTH, 25+BUTTONHEIGHT*(tid-1)+30+DAYHEIGHT, BUTTONWIDTH, BUTTONHEIGHT);
					buttons[dag][tid].setBackground(Color.WHITE);
					}

				buttons[dag][tid].addActionListener(createSchemaListener);
				schemalayoutPanel.add(buttons[dag][tid]);		
				schemalayoutPanel.add(dayTextArea[0][tid]);
				schemalayoutPanel.add(dayTextArea[dag][0]);
				}
			}
			
		schemalayoutPanel.add(currentWeek);
		schemalayoutPanel.add(prevWeek);
		schemalayoutPanel.add(nextWeek);
		schemalayoutPanel.add(dayTextArea[5][0]);
		schemalayoutPanel.add(dayTextArea[0][5]);
		schemalayoutPanel.setBorder(BorderFactory.createTitledBorder("Schema fr Sal TP1"));
		schemaAppPanel.setBorder(BorderFactory.createTitledBorder("Boka tidpunkt"));
		
		//Create schemaAppPanelListeners
		appOK.addActionListener(addAppointmentListener);
		appAbort.addActionListener(addAppointmentListener);
		appCancel.addActionListener(addAppointmentListener);
		appSubjectDropDown.addActionListener(addAppointmentListener);
		prevWeek.addActionListener(changeWeekListener);
		nextWeek.addActionListener(changeWeekListener);


	}
	
	/**
	*Method to create and open JPanel schemaAppPanel
	*Initiate Buttons, TextField, DropDown etc
	*Variables: Temporary appointment variables.
	*						are set to real variables when OK is pressed
	**/
	public void openSchemaAppPanel(int dag, int tid) {
			String teacher = "";
			String subject = "";
			String info = "";
			String teacherid = userHandler.currentUser.id;
							
			connect();
			try {
				ResultSet buttonInfo = stmt.executeQuery( "SELECT teachid, courseid, info FROM Schedule WHERE day='"+dag+"' AND time='"+tid+"' AND week='"+activeWeek+"';");
				while(buttonInfo.next()) {
					teacher = buttonInfo.getString("teachid");
					subject = buttonInfo.getString("courseid");
					info = buttonInfo.getString("info");
				}
				System.out.println(teacher + " _ " + teacherid + " _ " + subject + " _ " + info);					
			}
			catch (SQLException e) {
		 		System.out.println("Error: SQLException\n Error in SchemaAppPanel:\t" +e.getSQLState());
			}
			catch (Exception e) {
		 		System.out.println("Error: Exception\n Error in SchemaAppPanel:\t");
			}
			disconnect();
			
			
			if(userSubjects != null)
			{
				for(int y=0; y < userSubjects.length; y++)
				{
					appSubjectDropDown.removeItem(userSubjects[y]);
				}
			}
			userSubjects = userHandler.currentUser.getSubject();
			for(int x=0; x < userSubjects.length; x++)
			{
				appSubjectDropDown.addItem(userSubjects[x]);
			}
						
			schemaAppPanel.remove(appDay);
			schemaAppPanel.remove(appTime);
			schemaAppPanel.remove(appSubject);
			schemaAppPanel.remove(appToolTip);
			schemaAppPanel.remove(appSubjectDropDown);
			appSubjectDropDown.setEnabled(false);
			appSubjectDropDown.setVisible(false);
			schemaAppPanel.remove(appTextFormat);
			appTextFormat.setEnabled(false);
			appTextFormat.setVisible(false);
			schemaAppPanel.remove(appToolTipText);
			appToolTipText.setEnabled(false);
			appToolTipText.setVisible(false);
			schemaAppPanel.remove(appOK);
			appOK.setEnabled(false);
			appOK.setVisible(false);
			schemaAppPanel.remove(appAbort);
			appAbort.setEnabled(false);
			appAbort.setVisible(false);			
			appCancel.setEnabled(false);
			appCancel.setVisible(false);			
	
			appDay.setText("Dag: " + weekday[dag]);
			appTime.setText("Tid: " + daytime[tid]);
			//Skriver ut bokningsinformation om bokat
			if(subject != null) {
				appToolTip.setText("Info:");
				appToolTipText.setText(info);
				}
			//Annars skriv ut valmjligheterna
			else{
				appToolTip.setText("Info:");
				appToolTipText.setText("Maximalt 20 tecken");
			}
			//Skriver ut mne om bokat annars valmjligheterna
			if(subject != null)
				appSubject.setText("mne: "+subject);
			else
				appSubject.setText("mne:");

			if(subject.compareTo("") == 0 || teacher.compareTo(teacherid) == 0 || userHandler.currentUser.admin)
			{
				appSubjectDropDown.setEnabled(true);
				appSubjectDropDown.setVisible(true);
				appToolTipText.setEnabled(true);
				appToolTipText.setVisible(true);
				appOK.setEnabled(true);
				appOK.setVisible(true);
				appAbort.setEnabled(true);
				appAbort.setVisible(true);
				appTextFormat.setEnabled(true);
				appTextFormat.setVisible(true);
				if(subject != null) {
					appCancel.setEnabled(true);
					appCancel.setVisible(true);
				}
			}
		
			else
			{
				appSubjectDropDown.setEnabled(false);
				appSubjectDropDown.setVisible(false);
				appToolTipText.setEnabled(false);
				appToolTipText.setVisible(true);
				appOK.setEnabled(false);
				appOK.setVisible(false);
				appAbort.setEnabled(false);
				appAbort.setVisible(false);
				appTextFormat.setEnabled(false);
				appTextFormat.setVisible(false);
				appCancel.setEnabled(false);
				appCancel.setVisible(false);
			}

		//Add Compontents to Panel
		schemaAppPanel.add(appDay);
		schemaAppPanel.add(appTime);
		schemaAppPanel.add(appSubject);
		schemaAppPanel.add(appToolTip);
		schemaAppPanel.add(appSubjectDropDown);
		schemaAppPanel.add(appTextFormat);
		schemaAppPanel.add(appToolTipText);
		schemaAppPanel.add(appOK);
		schemaAppPanel.add(appAbort);
		schemaAppPanel.add(appCancel);
	}

	
	//Method to call when empyting button/slot
	public void makeEmptyButton(JButton button){
		String subject = "ledig";
		String tooltiptext = "ledig";
		String label = "ledig";			
								
		button.setText(subject);
		button.setToolTipText(tooltiptext);
		button.setBackground(Color.WHITE);
	}
	
	//Method to book classroom, day, hour
	public boolean cancelApp(int dag, int tid) {
		connect();
	
		try {
			if (stmt.executeQuery( "SELECT * FROM Schedule WHERE day="+dag+" AND time="+tid+" AND week="+activeWeek+";").next()) {
			String updateString = "DELETE FROM Schedule WHERE day="+dag+" AND time="+tid+" AND week="+activeWeek+";";
			System.out.println(updateString);
			stmt.executeUpdate(updateString);
			}
			
		}
		catch (SQLException e) {
	 		System.out.println("Error: SQLException\n Error in makeAppointment:\t" +e.getSQLState());
		}
		catch (Exception e) {
	 		System.out.println("Error: Exception\n Error in makeAppointment:\t" +e.getCause());
		}
		//disconnects from database (check below for code)
		disconnect();
		updateButtons();
		return true;
	}
	
	//Method to book classroom, day, hour
	public boolean makeAppointment(int dag, int tid, String label, String subject, String tooltiptext){
		connect();

		//tar reda p vilken kurs som r vald
		if (subject.compareTo("Matematik") == 0) {
			subject = "tng001";
		} else if (subject.compareTo("Engelska") == 0) {
			subject = "tng002";
		} else if (subject.compareTo("Fysik") == 0) {
			subject = "tng003";
		} else if (subject.compareTo("Svenska") == 0) {
			subject = "tng004";
		} else if (subject.compareTo("Kemi") == 0) {
			subject = "tng005";
		} else if (subject.compareTo("KomA") == 0) {
			subject = "tng006";
		}	

		try {
			if (stmt.executeQuery( "SELECT * FROM Schedule WHERE day="+dag+" AND time="+tid+" AND week="+activeWeek+";").next()) {
			String updateString = "UPDATE Schedule SET courseid='"+subject+"', info='"+tooltiptext+"', room='TP1', teachid='"+userHandler.getId()+"' WHERE day="+dag+" AND time="+tid+" AND week="+activeWeek+";";
			System.out.println(updateString);
			stmt.executeUpdate(updateString);
			}
			//hmtar info frn databas och lgger i rtt variabel.
			else {
			stmt.executeUpdate( "INSERT INTO Schedule (day, time, courseid, info, room, week, teachid) values ("+dag+", "+tid+", '"+subject+"','"+tooltiptext+"', 'TP1', "+activeWeek+", '"+userHandler.getId()+"');");
			}
		}
		catch (SQLException e) {
	 		System.out.println("Error: SQLException\n Error in makeAppointment:\t" +e.getSQLState());
		}
		catch (Exception e) {
	 		System.out.println("Error: Exception\n Error in makeAppointment:\t" +e.getCause());
		}
		//disconnects from database (check below for code)
		disconnect();
		updateButtons();
		return true;
	}

	//Enabele all buttons (except lunch)
	public void setEnable(boolean enable) {
		for (int dag = 0; dag < 5; dag++) {
			for (int tid = 0; tid < 5; tid++){
				if(tid != 2)
					buttons[dag][tid].setEnabled(enable);
			}
		}
	}
	
	
	/* FETUPPDATERAD FUNKTION
	 * Nu hmtar den allt frn databasen!
	 */
	public void updateButtons() {
		//stter alla knappar till tomma!
		//finns skert bttre stt.
		//gjorde bara s att det funkade
		for (int dag = 0; dag < 5; dag++) {
			for (int tid = 0; tid < 5; tid++){
				if(tid != 2) {
					makeEmptyButton(buttons[dag][tid]);			
	       		}
	       	}
		}
		//connect to database (check below for code)
		connect();
		try {
			//hmtar info frn databas och lgger i rtt variabel.
			ResultSet schedule = stmt.executeQuery( "SELECT time, day, week, info, room, Schedule.courseid, Schedule.teachid, coursename, teachname, teachcolor FROM Schedule, Courses, Teachers WHERE Courses.courseid=Schedule.courseid AND Schedule.teachid = Teachers.teachid AND week='"+activeWeek+"' ORDER BY week, day, time;");
			if(schedule.wasNull()) { System.out.println("Tomt");}
			else {
	    		while (schedule.next()) {
		        	int time = schedule.getInt("time");
		        	int day = schedule.getInt("day");
		        	int week = schedule.getInt("week");
		        	String tooltiptext = schedule.getString("info");
		       		String room = schedule.getString("room");
					String subject = schedule.getString("courseid");
					String human = schedule.getString("teachid");
					String coursename = schedule.getString("coursename");
					String teachname = schedule.getString("teachname");
					int teachcolor = schedule.getInt("teachcolor");
	
					//trimmar infon, s att den fr plats i knappen!
			        String label = tooltiptext;				
					label.trim();
					if(label.length() > 7)
						label = label.substring(0, 5).trim() + "...";
					
					//stter info p knapparna!					
					buttons[day][time].setText("<html>" + subject + "<br>" + label + "<br>" +human+"</html>");
					buttons[day][time].setHorizontalTextPosition(SwingConstants.LEFT);								
					buttons[day][time].setToolTipText(tooltiptext);
					ColorHSV color = new ColorHSV(teachcolor);
					buttons[day][time].setBackground(color.getLight());
					}
				}
		}
		catch (SQLException e) {
	 		System.out.println("Error: SQLException\n Error in schema:\t" +e.getSQLState());
		}
		catch (Exception e) {
	 		System.out.println("Error: Exception\n Error in schema2:\t");
		}
		//disconnects from database (check below for code)
		disconnect();
	}

	
	
	/**
	*Listeners fr att Klicka p Schemaknappar som sedan ppnar schemaAppPanel.
	**/
	ActionListener createSchemaListener = new ActionListener() {
		public void actionPerformed(ActionEvent e){
			for (int dag = 0; dag < 5; dag++){
				for(int tid = 0; tid < 5; tid++){
					if (e.getSource() == buttons[dag][tid]){
						day = dag;
						time = tid;
						openSchemaAppPanel(dag, tid);						
						schemaBokning.schemaApp(true);
						break;
					}
				}
			}
		}
	};
			
	/**
	*Listeners fr att Boka och Avboka
	**/
	ActionListener addAppointmentListener = new ActionListener() {
		public void actionPerformed(ActionEvent f){
			if (f.getSource() == appOK){
				schemaBokning.schemaApp(false);				//Close window
				/**
				*Change Temporary Variables
				*TempToolTipText and TempSubject
				*to real values
				**/
				if (appToolTipText.getText().compareTo("Maximalt 20 tecken") == 0)
					tempToolTipText = "";
				else
					tempToolTipText = appToolTipText.getText();
				
				tempSubject = (String)appSubjectDropDown.getSelectedItem();
				makeAppointment(day, time, tempToolTipText, tempSubject, tempToolTipText); //Add Appointment
			}
			if (f.getSource() == appCancel) {
				cancelApp(day, time);
				schemaBokning.schemaApp(false);				//Close panel + avbokning
				updateButtons();
			}
			if (f.getSource() == appAbort)
				schemaBokning.schemaApp(false);				//Only close panel
		}
	};
	
	/**
	*Listeners fr att visa Fregende och Nsta vecka
	**/
	ActionListener changeWeekListener = new ActionListener() {
		public void actionPerformed(ActionEvent g){
		
			if (g.getSource() == prevWeek){
				if (activeWeek == 1)
					activeWeek = 52;
				else
					activeWeek--;
					
				currentWeek.setText("Aktuell vecka: " + activeWeek);
				updateButtons();
			}
			if (g.getSource() == nextWeek){
				if (activeWeek == 52)
					activeWeek = 1;
				else
					activeWeek++;
					
				currentWeek.setText("Aktuell vecka: " + activeWeek);
				updateButtons();
			}

		}
	};
	private void connect() {
		try{
		
	   	Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
	   
	   	String url = "jdbc:odbc:torvald";
		con = DriverManager.getConnection(url,
	                     "tnm017+018_102", "tnm017+018_102A");
	    stmt = con.createStatement();
	    }
	    catch (SQLException e) {
     		System.out.println("Error: SQLException\n Error in connecting:\t" +e.getSQLState());
    	}
    	catch (ClassNotFoundException e) {
     		System.out.println("Error: ClassNotFoundException\n Error in connecting:\t");
    	}
    	catch (Exception e) {
     		System.out.println("Error: Exception\n Error in connecting:\t");
    	}
	}
  private void disconnect(){
  	try{
  	stmt.close();
    con.close();
    }
    catch (Exception e) {
     System.out.println("failed to disconnect");
    }
  }
}
